@@ -2,7 +2,7 @@
 /**
  *  @brief QMdiSubWindow for the analysis of data
  *
- *  Copyright 2015-2022 EDF-Phimeca
+ *  Copyright 2015-2023 EDF-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -47,9 +47,9 @@
 #endif
 
 #include <openturns/Normal.hxx>
+#include <openturns/DistFunc.hxx>
 
 #include <QVBoxLayout>
-#include <QScrollArea>
 #include <QHeaderView>
 #include <QSplitter>
 #include <QSortFilterProxyModel>
@@ -65,6 +65,7 @@ DataAnalysisWindow::DataAnalysisWindow(Item * item, QWidget * parent)
   , designOfExperiment_()
   , result_()
   , hasMaximumCV_(false)
+  , hasMaximumCILength_(false)
   , analysisStopCriteriaMessage_()
   , analysisErrorMessage_()
   , failedInputSample_()
@@ -253,6 +254,24 @@ void DataAnalysisWindow::addSummaryTab()
     valuesList << QString::number(maxCoefOfVariation);
   }
 
+  // - CI length
+  if (hasMaximumCILength_)
+  {
+    namesList << tr("Sample maximum CI length");
+    const UnsignedInteger nbInputs =  designOfExperiment_.getInputSample().getDimension();
+    const Scalar sqrtSampleSize = sqrt(designOfExperiment_.getInputSample().getSize());
+    Scalar maxCILength = 0.;
+    for (UnsignedInteger i = nbInputs; i < result_.getCoefficientOfVariation().getSize(); ++i)
+      if (result_.getStandardDeviation()[i].getSize() == 1)
+      {
+        Scalar CI = 2*result_.getStandardDeviation()[i][0]*DistFunc::qNormal(0.5*(1+levelConfidenceInterval_)) / sqrtSampleSize;
+        maxCILength = std::max(CI, maxCILength);
+      }
+    valuesList << QString::number(maxCILength);
+  }
+
+
+
   ParametersTableView * table = new ParametersTableView(namesList, valuesList, true, true);
   parametersGroupBoxLayout->addWidget(table);
   tabLayout->addWidget(parametersGroupBox, row, 0);
@@ -307,7 +326,7 @@ void DataAnalysisWindow::addPDF_CDFTab()
   // input indices
   Indices inInd(inputNames_.size());
   inInd.fill();
-  // ouput indices
+  // output indices
   Indices ind(outputNames_.size());
   ind.fill(inputNames_.size());
   // indices with good order
@@ -330,7 +349,7 @@ void DataAnalysisWindow::addPDF_CDFTab()
 
     // CDF
     PlotWidget * cdfPlot = new PlotWidget(tr("distributionCDF"));
-    cdfPlot->plotHistogram(designOfExperiment_.getSample().getMarginal(ind[i]), 1);
+    cdfPlot->plotHistogram(designOfExperiment_.getSample().getMarginal(ind[i]), PlotWidget::CDF);
     if (result_.getCDF()[ind[i]].getSize())
       cdfPlot->plotCurve(result_.getCDF()[ind[i]]);
     cdfPlot->setTitle(tr("CDF:") + " " + variablesNames[i]);
@@ -339,10 +358,25 @@ void DataAnalysisWindow::addPDF_CDFTab()
 
     stackedWidget->addWidget(cdfPlot);
 
+    // Survival Function, backwards compatibility
+    PlotWidget * survPlot = new PlotWidget(tr("distributionSurvivalFunction"));
+    if (result_.getSurvivalFunction().getSize())
+    {
+      survPlot->plotHistogram(designOfExperiment_.getSample().getMarginal(ind[i]), PlotWidget::Survival);
+      if (result_.getSurvivalFunction()[ind[i]].getSize())
+        survPlot->plotCurve(result_.getSurvivalFunction()[ind[i]]);
+      survPlot->setTitle(tr("SurvivalFunction:") + " " + variablesNames[i]);
+      survPlot->setAxisTitle(QwtPlot::xBottom, variablesAxisTitles[i]);
+      survPlot->setAxisTitle(QwtPlot::yLeft, tr("SurvivalFunction"));
+      stackedWidget->addWidget(survPlot);
+    }
+
     // Graph Setting
     QVector<PlotWidget*> listPlotWidgets;
     listPlotWidgets.append(pdfPlot);
     listPlotWidgets.append(cdfPlot);
+    if (result_.getSurvivalFunction().getSize())
+      listPlotWidgets.append(survPlot);
     PDFGraphSetting * graphSetting = new PDFGraphSetting(listPlotWidgets, PDFGraphSetting::Result, this);
     connect(graphSetting, SIGNAL(currentPlotChanged(int)), stackedWidget, SLOT(setCurrentIndex(int)));
 
@@ -372,7 +406,7 @@ void DataAnalysisWindow::addBoxPlotTab()
   // input indices
   Indices inInd(inputNames_.size());
   inInd.fill();
-  // ouput indices
+  // output indices
   Indices ind(outputNames_.size());
   ind.fill(inputNames_.size());
   // indices with good order
@@ -550,23 +584,25 @@ void DataAnalysisWindow::addTableTab()
   }
   else
   {
-    QTabWidget * tablesTabWidget = new QTabWidget;
+    tablesTabWidget_ = new QTabWidget;
 
     // tab with well evaluated points
-    tablesTabWidget->addTab(ExportableTableView::GetSampleTableViewWidget(getItem(), designOfExperiment_.getSample()), tr("Points"));
+    tablesTabWidget_->addTab(ExportableTableView::GetSampleTableViewWidget(getItem(), designOfExperiment_.getSample()), tr("Points"));
 
     // tab with failed points
     if (failedInputSample_.getSize())
     {
-      tablesTabWidget->addTab(ExportableTableView::GetSampleTableViewWidget(getItem(), failedInputSample_), tr("Failed blocks"));
+      tablesTabWidget_->addTab(ExportableTableView::GetSampleTableViewWidget(getItem(), failedInputSample_), tr("Failed points"));
+      if (errorDescription_.getSize())
+        addErrorTable();
     }
     // tab with not evaluated points
     if (notEvaluatedInputSample_.getSize())
     {
-      tablesTabWidget->addTab(ExportableTableView::GetSampleTableViewWidget(getItem(), notEvaluatedInputSample_), tr("Non-evaluated points"));
+      tablesTabWidget_->addTab(ExportableTableView::GetSampleTableViewWidget(getItem(), notEvaluatedInputSample_), tr("Non-evaluated points"));
     }
 
-    tabWidget_->addTab(tablesTabWidget, tr("Table"));
+    tabWidget_->addTab(tablesTabWidget_, tr("Table"));
   }
 }
 
@@ -592,21 +628,25 @@ void DataAnalysisWindow::addParaviewWidgetsTabs()
   else
   {
     // Table tab ------------------------------------------
-    QTabWidget * tablesTabWidget = new QTabWidget;
+    tablesTabWidget_ = new QTabWidget;
 
-    tablesTabWidget->addTab(PVSpreadSheetViewWidget::GetSpreadSheetViewWidget(pvSpreadSheetWidget, designOfExperiment_.getSample(), getItem()), tr("Table"));
+    tablesTabWidget_->addTab(PVSpreadSheetViewWidget::GetSpreadSheetViewWidget(pvSpreadSheetWidget, designOfExperiment_.getSample(), getItem()), tr("Table"));
 
     // -- failed points tab
     if (failedInSampleSize)
     {
       PVSpreadSheetViewWidget * failedPointsTable = new PVSpreadSheetViewWidget(this, PVServerManagerSingleton::Get());
-      tablesTabWidget->addTab(PVSpreadSheetViewWidget::GetSpreadSheetViewWidget(failedPointsTable, failedInputSample_, getItem()), tr("Failed points"));
+      tablesTabWidget_->addTab(PVSpreadSheetViewWidget::GetSpreadSheetViewWidget(failedPointsTable, failedInputSample_, getItem()), tr("Failed points"));
+      if (errorDescription_.getSize()) {
+        PVSpreadSheetViewWidget * errorTable = new PVSpreadSheetViewWidget(this, PVServerManagerSingleton::Get());
+        tablesTabWidget_->addTab(PVSpreadSheetViewWidget::GetSpreadSheetViewWidget(errorTable, failedInputSample_, getItem(), errorDescription_), tr("Error messages"));
+      }
     }
     // -- not evaluated points tab
     if (notEvalInSampleSize)
     {
       PVSpreadSheetViewWidget * notEvaluatedPointsTable = new PVSpreadSheetViewWidget(this, PVServerManagerSingleton::Get());
-      tablesTabWidget->addTab(PVSpreadSheetViewWidget::GetSpreadSheetViewWidget(notEvaluatedPointsTable, notEvaluatedInputSample_, getItem()), tr("Non-evaluated points"));
+      tablesTabWidget_->addTab(PVSpreadSheetViewWidget::GetSpreadSheetViewWidget(notEvaluatedPointsTable, notEvaluatedInputSample_, getItem()), tr("Non-evaluated points"));
     }
 
     // -- Parallel coordinates plot tab
@@ -640,7 +680,7 @@ void DataAnalysisWindow::addParaviewWidgetsTabs()
       // cobweb setting widget
       MultiPlotSettingWidget * cobwebSettingWidget = new MultiPlotSettingWidget(cobwebWidget, succeedAndFailedInS, succeedAndFailedInSRank, this);
 
-      tablesTabWidget->addTab(new WidgetBoundToDockWidget(cobwebWidget, cobwebSettingWidget, this), tr("Parallel coordinates plot"));
+      tablesTabWidget_->addTab(new WidgetBoundToDockWidget(cobwebWidget, cobwebSettingWidget, this), tr("Parallel coordinates plot"));
     }
 
     // -- scatter plots tab
@@ -696,10 +736,11 @@ void DataAnalysisWindow::addParaviewWidgetsTabs()
       // scatter plots setting widget
       ScatterSettingWidget * inSampleSettingWidget = new ScatterSettingWidget(sampleScatterPlotWidget, samples, rankSamples, this);
 
-      tablesTabWidget->addTab(new WidgetBoundToDockWidget(sampleScatterPlotWidget, inSampleSettingWidget, this), tr("Scatter plot"));
+      tablesTabWidget_->addTab(new WidgetBoundToDockWidget(sampleScatterPlotWidget, inSampleSettingWidget, this), tr("Scatter plot"));
     }
 
-    tabWidget_->addTab(tablesTabWidget, tr("Table"));
+    tabWidget_->addTab(tablesTabWidget_, tr("Table"));
+
   }
 
   // if only one variable or if only one point : do not need the following graphs
@@ -770,4 +811,28 @@ void DataAnalysisWindow::updateVariablesListVisibility(int indexTab)
 {
   variablesGroupBox_->setVisible(indexTab == 0 || indexTab == 1);
 }
+
+void DataAnalysisWindow::addErrorTable()
+{
+  ExportableTableView * errorTable = new ExportableTableView;
+  CustomStandardItemModel* errorTableModel = new CustomStandardItemModel(failedInputSample_.getSize()+1, failedInputSample_.getDimension()+2, errorTable);
+  errorTable->setModel(errorTableModel);
+
+  // header
+  for (UnsignedInteger j = 0; j < failedInputSample_.getDimension(); ++j)
+    errorTableModel->setNotEditableHeaderItem(0, j+1, failedInputSample_.getDescription()[j].c_str());
+  errorTableModel->setNotEditableHeaderItem(0, 0, tr("Row ID"));
+  errorTableModel->setNotEditableHeaderItem(0, failedInputSample_.getDimension()+1, tr("Error message"));
+
+  // Error Desc
+  for (UnsignedInteger i = 0; i < failedInputSample_.getSize(); ++i) {
+    for (UnsignedInteger j = 0; j < failedInputSample_.getDimension(); ++j)
+      errorTableModel->setNotEditableItem(i+1, j+1, failedInputSample_[i][j]);
+    errorTableModel->setNotEditableItem(i+1, failedInputSample_.getDimension()+1, errorDescription_[i].c_str());
+    errorTableModel->setNotEditableItem(i+1, 0, i);
+  }
+
+  errorTable->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
+  tablesTabWidget_->addTab(errorTable, tr("Error messages"));
+}
 }
